%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Exercice II du TD2 de "Système Robotiques Hétérogènes et Coopératifs" 
%%% 
%%% Ce script Matlab génére un graphe aléatoire binomial   
%%% avec n sommets selon le modèle de Erdös 
%%%
%%% Auteur: Fabio Morbidi
%%% Octobre 2022

clear
close all
clc

%% Paramètres du graphe aléatoire %%

n = 25;   %% Nombre de sommets du graphe

p = 0.05; %% Probabilité d'avoir une arête entre deux sommets

video = 1; %% Si video = 1, une animation est générée
           %% Si video = 0, aucune animation est générée 

%% La position des sommets du graphe est générée de façon aléatoire 
%% dans le carré [a, b] x [a, b] (distribution uniforme)                      
                       
a = -1; 
b = 1;

x = a + (b-a).*rand(n,1); %% Coordonnées x des sommets
y = a + (b-a).*rand(n,1); %% Coordonnées y des sommets

%%%%%%%%%%%%%%%%%

if video == 0
                      
%% Génération de la matrice d'adjacence du graphe aléatoire 

M = rand(n,n) < p; %% Les éléments de M sont tirés d'une distribution uniforme standard dans l'intervalle ouverte (0, 1)
Mu = triu(M,1);    %% "Symétrisation" de M: on extrait la partie triangulaire supérieure de M
A = Mu + Mu';      %% Matrice d'adjacence du graphe aléatoire

%%% Affichage du graphe aléatoire %%%

figure(1)
hold on
   
for i=1:n
   for j=1:n
       if A(i,j)==1
          line([x(i),x(j)],[y(i),y(j)],'color','k') %% Les arêtes du graphe aléatoire sont tracées 
       end
   end 
end

plot(x,y,'r.','MarkerSize',30) %% Les sommets du graphe aléatoire sont dessinés 

title(sprintf('Graphe aléatoire binomial G(n,p) avec :  n = %g,  p = %g',n,p))
xlabel('x')
ylabel('y')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% L'animation montre comment le graphe aléatoire évolue pour p qui varie
%%% entre 0 et 1 avec un pas q. Le nombre n de sommets est fixe. 

else

q = 0.05; %% Taille du pas    
    
figure(1)
hold on
for p = 0:q:1
    M = rand(n,n) < p; 
    Mu = triu(M,1); 
    A = Mu + Mu'; %% Matrice d'adjacence du graphe aléatoire
    for i=1:n
        for j=1:n
            if A(i,j)==1
               line([x(i),x(j)],[y(i),y(j)],'color','k') %% Les arêtes du graphe aléatoire sont tracées
            end
        end 
    end
    hold on
    plot(x,y,'r.','MarkerSize',30) %% Les sommets du graphe aléatoire sont dessinés
    xlabel('x')
    ylabel('y')
    title(sprintf('Graphe aléatoire binomial G(n,p) avec :  n = %g,  p = %g',n,p))
    pause(0.75)
    if p ~= 1
       clf
    end
end

end

%% EXERCICE: Vérifier numériquement les deux propriétés suivantes:
%% ----------------------------------------------------------------
%% La théorie des graphes de Erdös-Reyni, nous dit que:
%% 1) Une composante connexe "géante" devrait apparaître pour p > 1/(n-1) 
%% 2) Le graphe est censé devenir completement connexe quand p est proche de log(n)/n
